(***********************************************************************)
(*                                                                     *)
(*                           FaCiLe                                    *)
(*                 A Functional Constraint Library                     *)
(*                                                                     *)
(*            Nicolas Barnier, Pascal Brisset, LOG, CENA               *)
(*                                                                     *)
(* Copyright 2001 CENA. All rights reserved. This file is distributed  *)
(* under the terms of the GNU Lesser General Public License.           *)
(***********************************************************************)
(* $Id: fcl_misc.ml,v 1.10 2001/05/11 07:41:19 brisset Exp $ *)

let last_and_length l =
  let rec ll n = function
    [] -> Fcl_debug.internal_error "Fcl_misc.last_and_length: empty list"
  | [x] -> (x, n)
  | _::xs -> ll (n+1) xs
  in ll 1 l


let gen_int =
  let count = ref 0 in
  fun () -> incr count; !count
  
let int_overflow x =
  Fcl_debug.print_in_assert
    (float max_int > x && float min_int < x) "integer overflow\n"

module Operators = struct
  let ( * ) x y =
    assert (int_overflow (float x *. float y));
    x * y
      
  let (+) x y =
    assert (int_overflow (float x +. float y));
    x + y
      
  let (-) x y =
    assert (int_overflow (float x -. float y));
    x - y

  let (=+) x y = x := !x + y

  let min (a : int) b = if a < b then a else b
  let max (a : int) b = if a < b then b else a
end

let rec iter f n z =
  if n = 0 then z else f (iter f (n-1) z);;

let rec goedel f n z =
  if n = 0
  then z
  else f (n-1) (goedel f (n-1) z);;


let flags = (ref [] : (string * bool ref) list ref);;

let assoc_or_add n =
  try
    List.assoc n !flags
  with
    Not_found -> 
      let f = ref false in
      flags := (n, f) :: !flags;
      f;;

let protect name f =
  let already_in = assoc_or_add name in
  if !already_in then Fcl_debug.fatal_error (Printf.sprintf "%s not reentrant" name);
  already_in := true;
  try
    let x = f () in
      already_in := false;
    x
  with
    exc ->
      already_in := false;
      raise exc;;
	
